import pandas as pd
import pymongo
import spacy
import re
from langdetect import detect
from string import punctuation
import numpy as np
import os, sys

nlp = spacy.load('en_core_web_sm')  # type: object
resources_path = 'path to resources/resources/'
DEVELOPER_KEY = "ADD YOUR API KEY"
YOUTUBE_API_SERVICE_NAME = "youtube"
YOUTUBE_API_VERSION = "v3"


def get_channel_data(channel_id):
    print('Begin')
    youtube = build(YOUTUBE_API_SERVICE_NAME, YOUTUBE_API_VERSION, developerKey=DEVELOPER_KEY)
    max_results = 50

    parms = {
        'part': 'brandingSettings,contentDetails,contentOwnerDetails,id,invideoPromotion,localizations,snippet,statistics,status,topicDetails',
        'channelId': "UCHTK-2W11Vh1V4uwofOfR4w",
        'maxResults': max_results,
        'key': DEVELOPER_KEY
    }
    results = youtube.channels().list(
        part="statistics",
        id=channel_id,
    ).execute()
    return results


def openURL(self, url, parms):
    f = urlopen(url + '?' + urlencode(parms))
    data = f.read()
    f.close()
    matches = data.decode("utf-8")
    return matches


def extract_feature(video_text, channel_id):
    try:
        # load feature from database
        col_names = ["itemLength", "numWords", "numQuestionMark", "numExclamationMark", "containsQuestionMark",
                         "containsExclamationMark", "containsFirstOrderPron",
                         "containsSecondOrderPron", "containsThirdOrderPron", "numNegSentiWords",
                         "numPosSentiWords", "numSlangs", "hasColon", "numUppercaseChars", "numNouns", "channel_view_count", "channel_comment_count", "channel_subscriber_count", "channel_video_count"]
        print('open resources')
        # Open resources
        with open(resources_path + 'first-order-prons.txt') as f:
            df_pron_first = f.read().splitlines()

        with open(resources_path + 'second-order-prons.txt') as f:
            df_pron_second = f.read().splitlines()

        with open(resources_path + 'third-order-prons.txt') as f:
            df_pron_third = f.read().splitlines()

        with open(resources_path + 'slangwords-english.txt') as f:
            df_slang_english = f.read().splitlines()

        with open(resources_path + 'negative-words.txt') as f:
            df_senti_neg = f.read().splitlines()

        with open(resources_path + 'positive-words.txt') as f:
            df_senti_pos = f.read().splitlines()

            # language Spanish
        with open(resources_path + 'first-order-prons-spanish.txt') as f:
            df_pron_first_spa = f.read().splitlines()

        with open(resources_path + 'second-order-prons-spanish.txt') as f:
            df_pron_second_spa = f.read().splitlines()

        with open(resources_path + 'third-order-prons-spanish.txt') as f:
            df_pron_third_spa = f.read().splitlines()

        with open(resources_path + 'slangwords-spanish.txt') as f:
            df_slang_spanish = f.read().splitlines()

        with open(resources_path + 'negative-words-spanish.txt') as f:
            df_senti_neg_spanish = f.read().splitlines()

        with open(resources_path + 'positive-words-spanish.txt') as f:
            df_senti_pos_spanish = f.read().splitlines()

            # language German

            # language Spanish
        with open(resources_path + 'first-order-prons-german.txt') as f:
            df_pron_first_ger = f.read().splitlines()

        with open(resources_path + 'second-order-prons-german.txt') as f:
            df_pron_second_ger = f.read().splitlines()

        with open(resources_path + 'third-order-prons-german.txt') as f:
            df_pron_third_ger = f.read().splitlines()

        with open(resources_path + 'negative-words-german.txt') as f:
            df_senti_neg_german = f.read().splitlines()

        with open(resources_path + 'positive-words-german.txt') as f:
            df_senti_pos_german = f.read().splitlines()

        #print('Retrieved resources')

        df_feat = pd.DataFrame(columns=col_names)
        contains_first_order = 0
        contains_second_order = 0
        contains_third_order = 0
        number_slang = 0
        number_senti_neg = 0
        number_senti_pos = 0
        number_nouns = 0
        contain_explanations = 0
        contain_question = 0
        count_question_mark = 0
        count_explanation_mark = 0
        number_uppercase = 0
        has_colon = 0
        number_of_words = 0


        item_length = len(video_text)
        if item_length > 0:
            # detect language
            video_text = remove_urls(video_text)
            if len(video_text) > 0:
                try:
                    lan = detect(video_text)
                except:
                    lan = 'none'
            else:
                lan = 'none'

            count_question_mark = video_text.count("?")
            count_explanation_mark = video_text.count("!")

            if video_text.count("?") > 0:
                contain_explanations = 1

            if video_text.count("!") > 0:
                contain_question = 1

            if video_text.count(':') > 0:
                has_colon = 1

            # number Uppercase
            for c in video_text:
                if c.isupper():
                    number_uppercase += 1

            translation = video_text.maketrans("", "", punctuation)
            video_text_filtered = video_text.translate(translation)
            doc = nlp(video_text_filtered)
            number_of_words = len(doc)

            nouns = {'NN', 'NNS', 'NNP', 'NNPS'}
            for token in doc:

                if token.tag_ in nouns:
                    number_nouns += 1

                # slang
                if lan == 'en':
                    if token.text.lower() in df_pron_first:
                        contains_first_order = 1
                    if token.text.lower() in df_pron_second:
                        contains_second_order = 1
                    if token.text.lower() in df_pron_third:
                        contains_third_order = 1
                    if token.text.lower() in df_slang_english:
                        number_slang = number_slang + 1
                    if token.text.lower() in df_senti_neg:
                        number_senti_neg = number_senti_neg + 1
                    if token.text.lower() in df_senti_pos:
                        number_senti_pos = number_senti_pos + 1

                if lan == 'es':
                    if token.text.lower() in df_pron_first_spa:
                        contains_first_order = 1
                    if token.text.lower() in df_pron_second_spa:
                        contains_second_order = 1
                    if token.text.lower() in df_pron_third_spa:
                        contains_third_order = 1
                    if token.text.lower() in df_slang_spanish:
                        number_slang = number_slang + 1
                    if token.text.lower() in df_senti_neg_spanish:
                        number_senti_neg = number_senti_neg + 1
                    if token.text.lower() in df_senti_pos_spanish:
                        number_senti_pos = number_senti_pos + 1

                if lan == 'de':
                    if token.text.lower() in df_pron_first_ger:
                        contains_first_order = 1
                    if token.text.lower() in df_pron_second_ger:
                        contains_second_order = 1
                    if token.text.lower() in df_pron_third_ger:
                        contains_third_order = 1
                    if token.text.lower() in df_senti_neg_german:
                        number_senti_neg = number_senti_neg + 1
                    if token.text.lower() in df_senti_pos_german:
                        number_senti_pos = number_senti_pos + 1

        else:
            print('Video text length is not enough')
        # connect to youtube api in order to retrieve the channel statistics
        # API key is required
        channel_metadata = youtube_channel.get_channel_data(channel_id)
        features = {#'_id': video_id,
                        'itemLength': item_length,
                        'numWords': number_of_words,
                        'numQuestionMark': count_question_mark,
                        'numExclamationMark': count_explanation_mark,
                        'containsQuestionMark': contain_question,
                        'containsExclamationMark': contain_explanations,
                        'containsFirstOrderPron': contains_first_order,
                        'containsSecondOrderPron': contains_second_order,
                        'containsThirdOrderPron': contains_third_order,
                        'numNegSentiWords': number_senti_neg,
                        'numPosSentiWords': number_senti_pos,
                        'numSlangs': number_slang,
                        'hasColon': has_colon,
                        'numUppercaseChars': number_uppercase,
                        'numNouns': number_nouns,
                        'channel_view_count': int(channel_metadata['items'][0]['statistics']['viewCount']),
                        'channel_comment_count': int(channel_metadata['items'][0]['statistics']['commentCount']),
                        'channel_subscriber_count': int(channel_metadata['items'][0]['statistics']['subscriberCount']),
                        'channel_video_count': int(channel_metadata['items'][0]['statistics']['videoCount'])}

        df_feat.loc[len(df_feat)] = features
        scaler_data_max_ = np.load("scaler_max_yt.npy")
        scaler_data_min_ = np.load("scaler_min_yt.npy")

        feat = np.array(df_feat.values)
        scl = (feat - scaler_data_min_) / (scaler_data_max_ - scaler_data_min_)
        return scl

    except Exception as e:
        print('Extection in Feature extraction')
        exc_type, exc_obj, exc_tb = sys.exc_info()
        fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
        print(exc_type, fname, exc_tb.tb_lineno)
        print("Cannot Open URL or Fetch comments at a moment " + str(e))
        return np.empty(shape=[0, 0])

def remove_urls(vtext):
    vtext = re.sub(r'(https|http)?:\/\/(\w|\.|\/|\?|\=|\&|\%)*\b', '', vtext, flags=re.MULTILINE)
    return (vtext)


if __name__ == "__main__":
    extract_feature(database)
