import pandas as pd

data_file_path = '/path to input files/Input_data/'
model_path = '/path to code/Code/'


def load_metadata():
    # load responses
    path_meta_file_real = data_file_path + 'yt_meta.json'
    yt_df_meta = pd.read_json(path_or_buf=path_meta_file_real)

    # load cascade vs video id correspondences
    path_train_ids = data_file_path + 'cascade_ids_all.txt'
    f_train_ids = open(path_train_ids, "r")
    train_ids = f_train_ids.read().splitlines()

    ### TRAINING SET #####

    # load train indices
    path_train_index = data_file_path + 'train_idx.txt'
    f_train_index = open(path_train_index, "r")
    train_index = f_train_index.read().splitlines()

    # Get ids of video of the training set (train_video_ids)
    train_video_ids = []
    for row in train_ids:
        # print(row)
        cascade_id = row.split('\t')[1]
        video_id = row.split('\t')[0]
        # print(video_id)
        if cascade_id in train_index:
            print('Cascade {} is in training set with video {} '.format(cascade_id, video_id))
            train_video_ids.append(video_id)

    # filter the dataframe containing the features of all videos in order to retrieve those that serve as training samples
    train_data_responses = yt_df_meta[yt_df_meta['_id'].isin(train_video_ids)]
    # Load any information that want to process e.g. video_title - train_data_responses['video_title']

    train_labels = yt_df_meta['label']

    print('Train metadata')
    print(train_data_responses)
    print('Train Labels')
    print(train_labels)

    ################# TESTING ######################
    path_test_index = data_file_path + 'test_idx.txt'
    f_test_index = open(path_test_index, "r")
    test_index = f_test_index.read().splitlines()

    # Get ids of video of the testing set (test_video_ids)
    test_video_ids = []
    for row in train_ids:
        # print(row)
        cascade_id = row.split('\t')[1]
        video_id = row.split('\t')[0]
        # print(video_id)
        if cascade_id in test_index:
            print('Cascade {} is in training set with video {} '.format(cascade_id, video_id))
            test_video_ids.append(video_id)

    # filter the dataframe containing the features of all videos in order to retrieve those that serve as testing samples
    test_data_responses = yt_df_meta[yt_df_meta['_id'].isin(test_video_ids)]
    test_labels = yt_df_meta['label']
    print('Test metadata')
    print(test_data_responses)
    print('Test Labels')
    print(test_labels)


def load_features():
    # load responses
    path_meta_file_real = data_file_path + 'yt_meta.json'
    yt_df_meta = pd.read_json(path_or_buf=path_meta_file_real)

    # load cascade vs video id correspondences
    path_train_ids = data_file_path + 'cascade_ids_all.txt'
    f_train_ids = open(path_train_ids, "r")
    train_ids = f_train_ids.read().splitlines()

    # load features
    path_feature_file = data_file_path + 'yt_vf.csv'
    yt_df_features = pd.read_csv(path_feature_file)

    ### TRAINING SET #####

    # load train indices
    path_train_index = data_file_path + 'train_idx.txt'
    f_train_index = open(path_train_index, "r")
    train_index = f_train_index.read().splitlines()

    # Get ids of video of the training set (train_video_ids)
    train_video_ids = []
    for row in train_ids:
        # print(row)
        cascade_id = row.split('\t')[1]
        video_id = row.split('\t')[0]
        # print(video_id)
        if cascade_id in train_index:
            print('Cascade {} is in training set with video {} '.format(cascade_id, video_id))
            train_video_ids.append(video_id)

    # filter the dataframe containing the features of all videos in order to retrieve those that serve as training samples
    train_data_features = yt_df_features[yt_df_features['id'].isin(train_video_ids)]

    # Get the features
    train_data = train_data_features[
        ["item_length", "num_words", "contains_questionmark", "contains_exclamationmark", "contains_first_order_pronoun",
         "contains_second_order_pronoun", "contains_third_order_pronoun", "num_uppercasechars", "num_neg_sentiment_words",
         "num_pos_sentiment_words", "num_slangs", "has_colon", "num_exclamationmark", "num_questionmark", 'channel_video_count',
         'channel_view_count', 'channel_subscriber_count',
         'channel_comment_count']]
    # Convert to integers the boolean values
    train_data = train_data.astype(int)

    # Get labels: 1 for fake , 0 for real videos
    train_labels = train_data_features['label']

    print('Train features')
    print(train_data)
    print('Train labels')
    print(train_labels)

    ################# TESTING ######################
    path_test_index = data_file_path + 'test_idx.txt'
    f_test_index = open(path_test_index, "r")
    test_index = f_test_index.read().splitlines()

    # Get ids of video of the testing set (test_video_ids)
    test_video_ids = []
    for row in train_ids:
        # print(row)
        cascade_id = row.split('\t')[1]
        video_id = row.split('\t')[0]
        # print(video_id)
        if cascade_id in test_index:
            print('Cascade {} is in training set with video {} '.format(cascade_id, video_id))
            test_video_ids.append(video_id)

    # filter the dataframe containing the features of all videos in order to retrieve those that serve as testing samples
    test_data_features = yt_df_features[yt_df_features['id'].isin(test_video_ids)]

    test_data = test_data_features[
        ["item_length", "num_words", "contains_questionmark", "contains_exclamationmark", "contains_first_order_pronoun",
         "contains_second_order_pronoun", "contains_third_order_pronoun", "num_uppercasechars", "num_neg_sentiment_words",
         "num_pos_sentiment_words", "num_slangs", "has_colon", "num_exclamationmark", "num_questionmark", 'channel_video_count',
         'channel_view_count', 'channel_subscriber_count',
         'channel_comment_count']]

    test_data = test_data.astype(int)
    test_labels = test_data_features['label']

    print('Test features')
    print(test_data)
    print('Test labels')
    print(test_data)



if __name__ == "__main__":
    # load the extracted textual features of the training and testing set
    load_features()

    # load the metadata responses of the training and testing set (video title, desciption, channel created time etc.)
    load_metadata()

