import pandas as pd
import os
from sklearn import svm
from sklearn.externals import joblib
import numpy as np

data_file_path = '/path to input files/Input_data/'
model_path = '/path to code/Code/'


def main():
    # load responses
    path_meta_file_real = data_file_path + 'yt_meta.json'
    yt_df_meta = pd.read_json(path_or_buf=path_meta_file_real)

    # load train indices
    path_train_index = data_file_path + 'train_idx.txt'
    f_train_index = open(path_train_index, "r")
    train_index = f_train_index.read().splitlines()

    # load cascade vs video id correspondences
    path_train_ids = data_file_path + 'cascade_ids_all.txt'
    f_train_ids = open(path_train_ids, "r")
    train_ids = f_train_ids.read().splitlines()

    # load features
    path_feature_file = data_file_path + 'yt_vf.csv'
    yt_df_features = pd.read_csv(path_feature_file)

    # Get ids of video of the training set (train_video_ids)
    train_video_ids = []
    for row in train_ids:
        # print(row)
        cascade_id = row.split('\t')[1]
        video_id = row.split('\t')[0]
        # print(video_id)
        if cascade_id in train_index:
            print('Cascade {} is in training set with video {} '.format(cascade_id, video_id))
            train_video_ids.append(video_id)

    # filter the dataframe containing the features of all videos in order to retrieve those that serve as training samples
    train_data_features = yt_df_features[yt_df_features['id'].isin(train_video_ids)]

    # Get the features
    train_data = train_data_features[
        ["item_length", "num_words", "contains_questionmark", "contains_exclamationmark", "contains_first_order_pronoun",
         "contains_second_order_pronoun", "contains_third_order_pronoun", "num_uppercasechars", "num_neg_sentiment_words",
         "num_pos_sentiment_words", "num_slangs", "has_colon", "num_exclamationmark", "num_questionmark", 'channel_video_count',
         'channel_view_count', 'channel_subscriber_count',
         'channel_comment_count']]
    # Convert to integers the boolean values
    train_data = train_data.astype(int)

    # Get labels: 1 for fake , 0 for real videos
    train_labels = train_data_features['label']

    # Normalize feature values
    scaler_data_max_ = []
    scaler_data_min_ = []
    result = train_data.copy()
    for feature_name in train_data.columns:
        max_value = train_data[feature_name].max()
        min_value = train_data[feature_name].min()
        print(f' Manx value {max_value} and Min value {min_value} of {feature_name}')
        scaler_data_max_ = np.append(scaler_data_max_, max_value)
        scaler_data_min_ = np.append(scaler_data_min_, min_value)
        result[feature_name] = (train_data[feature_name] - min_value) / (max_value - min_value)

    # Save scaler in order to user it for normalizing the values of the test set
    if not os.path.isfile(model_path + 'scaler_max.npy'):
        np.save(model_path + 'scaler_max.npy', scaler_data_max_)
        np.save(model_path + 'scaler_min.npy', scaler_data_min_)

    # Train classifier
    clf = svm.SVC(C=100000, gamma=0.01, probability=True)
    clf.fit(result.values, train_labels)
    print(clf)

    filename = model_path + 'svm_model.sav'
    if not os.path.isfile(filename):
        joblib.dump(clf, filename)

    ################# TESTING ######################
    path_test_index = data_file_path + 'test_idx.txt'
    f_test_index = open(path_test_index, "r")
    test_index = f_test_index.read().splitlines()

    # Get ids of video of the testing set (test_video_ids)
    test_video_ids = []
    for row in train_ids:
        # print(row)
        cascade_id = row.split('\t')[1]
        video_id = row.split('\t')[0]
        # print(video_id)
        if cascade_id in test_index:
            print('Cascade {} is in training set with video {} '.format(cascade_id, video_id))
            test_video_ids.append(video_id)

    # filter the dataframe containing the features of all videos in order to retrieve those that serve as testing samples
    test_data_features = yt_df_features[yt_df_features['id'].isin(test_video_ids)]

    test_data = test_data_features[
        ["item_length", "num_words", "contains_questionmark", "contains_exclamationmark", "contains_first_order_pronoun",
         "contains_second_order_pronoun", "contains_third_order_pronoun", "num_uppercasechars", "num_neg_sentiment_words",
         "num_pos_sentiment_words", "num_slangs", "has_colon", "num_exclamationmark", "num_questionmark", 'channel_video_count',
         'channel_view_count', 'channel_subscriber_count',
         'channel_comment_count']]

    test_data = test_data.astype(int)
    test_labels = test_data_features['label']

    scl = (test_data - scaler_data_min_) / (scaler_data_max_ - scaler_data_min_)
    if not os.path.isfile(filename):
        print('loaded model')
        loaded_model = joblib.load(filename)
        result = loaded_model.score(scl, test_labels.values)
    else:
        print('trained model')
        result = clf.score(scl, test_labels.values)

    print(result)


if __name__ == "__main__":
    main()
